<?php


include_spip('inc/dolispip');


/**
 * Get orders list
 *
 */
function dolispip_get_list_commandes() {
    $endpoint = 'orders';
    return getDolibarrApi($endpoint, array(), $use_user, false);
}

/**
 * Récupère les infos d'une commande
 * 
 * @param int $idCommande
 * @param boolean $use_user
 * @return boolean|mixed|string
 */
function dolispip_get_infos_commande($idCommande, $use_user=false) {
    $endpoint = 'orders/'.$idCommande;
    $res = getDolibarrApi($endpoint, array(), $use_user);
    return $res;
}

/**
 * Récupère les lignes d'une commande
 * 
 * @param int $idCommande
 * @param boolean $use_user
 * @return boolean|mixed|string
 */
function dolispip_get_lines_commande($idCommande, $use_user=false) {
    $endpoint = "orders/$idCommande/lines";
    $res = getDolibarrApi($endpoint);
    return $res;
}

/**
 * Création d'une commande avec l'API Dolibarr
 * 
 * @param array $order  Tableau avec les infos de commande
 */
function dolispip_creer_cde($order) 
{
    $endpoint = 'orders';
    $res = postDolibarrApi($endpoint, $order, true);
    return $res;
}


/**
 * Valide une commande
 *
 */
function dolispip_valid_cde($id) 
{
    $endpoint = "orders/$id/validate";
    $res = postDolibarrApi($endpoint, array(), true);
    
    return $res;
}

/*
 * Ferme une commande 
 */
function dolispip_facturer_cde($id) 
{
    include_spip('inc/config');
    $WS_DOL_URL = lire_config('dolispip/dolibarr_url').'webservices/server_order.php?wsdl';

    $WS_METHOD  = 'getOrder';
    $params = array('id'=>$id);
    $result = doSoapCall($WS_METHOD, $params, $WS_DOL_URL, 0, 1);
    if (! $result &&  $result['result_code'] != 'OK') {
        var_dump($soapclient);
        print '<h2>Erreur SOAP 1</h2>'.$soapclient->error_str;
        return false;
    } else {
        $order = $result['order'];
        $order->status = 3;
        $order->billed = true;

        //getOrder miss type value, we enforce this
        foreach($order->lines as $line) {
            $line->type = 1;
        }

        $WS_METHOD  = 'updateOrder';
        $params = array('order'=>$order);

        $result = doSoapCall($WS_METHOD, $params, $WS_DOL_URL);
        if (! $result &&  $result['result_code'] != 'OK') {
            var_dump($soapclient);
            print '<h2>Erreur SOAP 1</h2>'.$soapclient->error_str;
            return false;
        }
        return true;
    }
}


/*
 * Charge le tableau d'un objet SOAP order
*/
function load_doliorder_array() 
{
    $order = array();

    $order['id'] = '0';
    $order['ref']='';
    $order['ref_client']='';
    $order['ref_ext']="";
    $order['ref_int']="";
    $order['socid'] = '';
    $order['statut'] = '0'; // 0 pour pouvoir ajouter les lignes, à passer à 1 après la création
    $order['facturee']="0";
    $order['billed']="0"; // Since 3.8

    $order['total_ht'] = '';
    $order['total_tva'] = '';
    $order['total_localtax1'] = '';
    $order['total_localtax2'] = '';
    $order['total_ttc'] = '';

    $order['date'] = '';
    $order['date_commande'] = '';
    $order['date_livraison'] = '';
    

    $order['remise'] = '';
    $order['remise_percent'] = '';
    $order['remise_absolue'] = '';

    $order['source'] = '';
    $order['note_private'] = '';
    $order['note_public'] = '';
    $order['project_id'] = '';

    $order['mode_reglement_id'] = ''; //
    $order['mode_reglement_code'] = '';
    $order['mode_reglement'] = "";

    $order['cond_reglement_id'] = "1"; // A réception
    $order['cond_reglement_code'] = '';
    $order['cond_reglement'] = "";
    $order['cond_reglement_doc'] = "";

    $order['date_livraison'] = '';
    $order['fk_delivery_address'] = '';
    $order['demand_reason_id'] = "1"; // Internet

    $order['lines'] = array();

    return $order;
}

function load_line_doliorder_array() 
{
    $line = array();

    $line['id'] = '';
    $line['type'] = '';
    $line['fk_commande'] = '';
    $line['fk_parent_line'] = '';
    $line['desc'] = '';
    $line['qty'] = '';
    $line['price'] = '';
    $line['unitprice'] = '';
    $line['vat_rate'] = '';
    $line['remise'] = '';
    $line['remise_percent'] = '';
    $line['total_net'] = '';
    $line['total_vat'] = '';
    $line['total'] = '';
    $line['date_start'] = '';
    $line['date_end'] = '';
    $line['product_id'] = '';
    $line['product_ref'] = '';
    $line['product_label'] = '';
    $line['product_desc'] = '';

    return $line;
}
