<?php
/***************************************************************************\
 *  SPIP, Systeme de publication pour l'internet                           *
 *                                                                         *
 *  Copyright (c) 2001-2008                                                *
 *  Arnaud Martin, Antoine Pitrou, Philippe Riviere, Emmanuel Saint-James  *
 *                                                                         *
 *  Ce programme est un logiciel libre distribue sous licence GNU/GPL.     *
 *  Pour plus de details voir le fichier COPYING.txt ou l'aide en ligne.   *
\***************************************************************************/

if (!defined("_ECRIRE_INC_VERSION")) return;

/*
 * Fonction d'appel d'un webservice Dolibarr
 *
 * $method : méthode du webservice
 * $params : tableau de parametre ex : array('id' => '9');
 * $url : url du webservice
 * $useUser : s'il faut utiliser l'utilisateur connecté pour appeler les webservices (non par défaut, utilisation de l'utilisateur défini dans la configuration du module)
 * $retourBrut : indique si la fonction doit traiter le retour du webservice (par défaut) ou retourner le résultat brut de la requête
 */
function doSoapCall($method, $params, $url, $useUser=false, $retourBrut=false) {
	include_spip('inc/session');
	include_spip('inc/config');
	spip_log("Appel méthode $method $url ","dolispip");
	$soapclient = new SoapClient($url,
		array(
			'wsdl_cache' => lire_config('dolispip/activer_cache_soap'),
			'trace'=> lire_config('dolispip/activer_debug')
		)
	);

	if ($soapclient)
	{
		$soapclient->__setLocation($url);
		$soapclient->soap_defencoding='UTF-8';
	}
	else return false;

	$authentication = getAuthentificationDolibarrWS($useUser);

	$parameters = array('authentication'=>$authentication);
	$parameters = array_merge($parameters,$params);

	$result = $soapclient->__call($method,$parameters);

	// DEBUG
	if(lire_config('dolispip/activer_debug') == "1" && session_get('statut') == '0minirezo')
	{
		print '<div class="notice"><h3>::dolispip:: '.$method.'</h3><pre>';print_r($result);print $method;print ' - login : '.$authentication['login'].'</pre></div>';
	}

	if (! $result)
	{
		spip_log("Erreur webservice méthode $method","dolispip");

		return false;
	}

	// On veut le retour brut
	if($retourBrut)
	{
		return $result;
	}
	else
	{
		if(is_array($result))
		{

			// Requête ok
			if($result['result']->result_code == 'OK')
			{
				unset($result['result']);
				return $result;
			}
			elseif($result['result']->result_code == 'PERMISSION_DENIED' OR $result['result']->result_code == 'ERROR_FETCH_USER')
			{
				// Redirection si l'utilisateur n'est pas autorisé à voir l'objet
				$redirect = generer_url_public('403','',true,0);
				include_spip('inc/headers');
				if(!lire_config('dolispip/activer_debug'))
					redirige_par_entete($redirect);
				return '';
			}
			elseif ($result['result']->result_code == 'BAD_CREDENTIALS')
			{
				// Redirection vers page login
				$redirect = generer_url_public('moncompte','',true,0);
				include_spip('inc/headers');
				if(!lire_config('dolispip/activer_debug'))
					redirige_par_entete($redirect);
				return '';
			}
			elseif($result['result']->result_code == 'BAD_PARAMETERS')
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}
	return false;
}

/*
 * Construction du tableau pour l'authentification au webservice Dolibarr
 * Si $useUser=true alors on authentifie avec l'utilisateur courant
 */
function getAuthentificationDolibarrWS($useGenericUser=false) {
	include_spip('inc/session');
	include_spip('inc/config');
	$dolibarr_key = lire_config('dolispip/dolibarr_key');
	$sourceapplication = 'SPIP';

	if($useGenericUser) {
		$login = session_get('login');
		$password = session_get('doli_pass');
	}
	else {
		// Appel du WS avec utilisateur générique
		$login = lire_config('dolispip/dolibarr_user');
		$password = lire_config('dolispip/dolibarr_pass');
	}

	$authentication=array(
		'dolibarrkey'=>$dolibarr_key,
		'sourceapplication'=>$sourceapplication,
		'login'=>$login,
		'password'=>$password,
		'entity'=>'1'
	);

    return $authentication;
}

function initDolibarrApi($parameters, $useGenericUser)
{
	$url_api = lire_config('dolispip/dolibarr_url') . 'api/index.php/';
	$api = new DolispipRestClient([
    'base_url' => $url_api,
    //'format' => "json",
    'headers' => setHeadersApi($parameters, $useGenericUser),
	]);
	//unset($parameters['api_key']);

	return $api;
}
/**
 * Get a resource from Dolibarr API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 *
 * @return boolean|mixed|string
 */
function getDolibarrApi($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {

	include_spip('lib/DolispipRestclient');

	$api = initDolibarrApi($parameters, $useGenericUser);

	$result = $api->get($endpoint, $parameters);
	// TODO : debug mode
	//var_dump($result); exit;
	return returnApiCall($result, $retourBrut);
}


/**
 * Put a resource from Dolibarr API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 *
 * @return boolean|mixed|string
 */
function putDolibarrApi($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {

	include_spip('lib/DolispipRestclient');

	$api = initDolibarrApi($parameters, $useGenericUser);

	$result = $api->put($endpoint, $parameters);
	//var_dump($result); exit;
	return returnApiCall($result, $retourBrut);
}


/**
 * Post a resource from Dolibarr API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 *
 * @return boolean|mixed|string
 */
function postDolibarrApi($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {

	include_spip('lib/DolispipRestclient');

	$api = initDolibarrApi($parameters, $useGenericUser);

	$result = $api->post($endpoint, $parameters);
	//var_dump($result); exit;
	return returnApiCall($result, $retourBrut);
}

/**
 * Delete a resource from Dolibarr API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 *
 * @return boolean|mixed|string
 * @uses initDolibarrApi
 */
function deleteDolibarrApi($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {

	include_spip('lib/DolispipRestclient');

	$api = initDolibarrApi($parameters, $useGenericUser);

	$result = $api->delete($endpoint, $parameters);
	//var_dump($result); exit;
	return returnApiCall($result, $retourBrut);
}

function setHeadersApi($parameters, $useGenericUser=false)
{
		if(! isset($parameters['api_key'])) {
			$parameters['api_key'] = true;
		}
		if( $parameters['api_key'] === false) {
			unset($parameters['api_key']);
		}

		// We have api_key into parameters array, we add API key
		if (isset($parameters['api_key']) || $parameters['api_key'] === true) {
			$parameters['api_key'] = getDolispipRestToken($useGenericUser);
		}

		// Add DOLAPIKEY into header
		if ($parameters['api_key']) {
			$curlHeaders['DOLAPIKEY'] = $parameters['api_key'];
		}



		return $curlHeaders;

}

function returnApiCall($result, $retourBrut = false)
{
	//var_dump($result); exit;
	if(lire_config('dolispip/activer_debug') && session_get('statut') == '0minirezo') {
		var_dump($result);
	}
	// On veut le retour brut
	if($retourBrut) {
		return $result;
	}

	$retour = '';
	switch($result->info->http_code) {

		case 200:
				$retour = $result->decode_response();
				break;
		case "400":
				$retour = false;
				break;

		case "401":
				// Redirection vers page login
				$redirect = generer_url_public('login','',true,0);
				include_spip('inc/headers');
				//if(!lire_config('dolispip/activer_debug'))
				//	redirige_par_entete($redirect);
				$retour = false;
				break;
		case "403":
		    // Redirection vers page 403
		    //$redirect = generer_url_public('403','',true,0);
		    //include_spip('inc/headers');
		    //if(!lire_config('dolispip/activer_debug'))
		    //    redirige_par_entete($redirect);
	        $retour = '';
	        break;

	}
	return $retour;
}

/**
 * Format URL for API
 * Used by doRest* function
 * @deprecated
 *
 */
function dolispipFormatApiUrl($endpoint, $params = array(), $useGenericUser=false) {

    // API URL from dolispip core plugin
    $url_api = lire_config('dolispip/dolibarr_url') . 'api/index.php/';
    $url_api .= $endpoint;

    unset($params['api_key']);
    // Format and return URL
    if (count($params) > 1) {
        $url_api .= '?' . http_build_query($params);
    }
    return $url_api;
}

/**
 * Do a request on Dolibarr API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 *
 * @return boolean|mixed|string
 * @deprecated use getDolibarrApi() instead
 */
function doRestCall($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {
	if(! isset($parameters['api_key'])) {
		$parameters['api_key'] = '';
	}
	if( $parameters['api_key'] === false) {
		unset($parameters['api_key']);
	}

	// Format URL
	$url_api = dolispipFormatApiUrl($endpoint, $parameters, $useGenericUser);
	$curlHeaders = array();
	$curl = curl_init($url_api);
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

	// Ask json content
	$curlHeaders[] = 'Content-type: application/json';

    // We have api_key into parameters array, we add API key
    if (isset($parameters['api_key']) || $parameters['api_key'] === true) {
        $parameters['api_key'] = getDolispipRestToken($useGenericUser);
    }

    // Add DOLAPIKEY into header
    if ($parameters['api_key']) {
        $curlHeaders[] = 'DOLAPIKEY: ' . $parameters['api_key'];
    }
    curl_setopt($curl, CURLOPT_HTTPHEADER, $curlHeaders);

    // Send request
	$curl_response = curl_exec($curl);
	if ($curl_response === false) {
	    curl_getinfo($curl);
	    curl_close($curl);
	    //die('error occured during curl exec. Additioanl info: ' . var_export($info));
	    return false;
	}

	curl_close($curl);

	// On veut le retour brut
	if($retourBrut) {
		return $curl_response;
	}
	if(!$retourBrut)	{
		// Decode and convert into array
		$retour = json_decode($curl_response, true);

		if(isset($retour['error']) && isset($retour['error']['code'])) {
			switch($retour['error']['code']) {
				case "400":
					$retour = false;
					break;

				case "401":
					// Redirection vers page login
					$redirect = generer_url_public('moncompte','',true,0);
					include_spip('inc/headers');
					if(!lire_config('dolispip/activer_debug'))
						redirige_par_entete($redirect);
					$retour = '';
					break;
				case "403":
				    // Redirection vers page 403
				    $redirect = generer_url_public('403','',true,0);
				    include_spip('inc/headers');
				    if(!lire_config('dolispip/activer_debug'))
				        redirige_par_entete($redirect);
				        $retour = '';
				        break;
			}
		}
		return $retour;
	}
}


/**
 * Do a POST request to the REST API
 *
 * @param string $endpoint
 * @param array $parameters
 * @param bool $useGenericUser
 * @param bool $retourBrut
 * @return boolean|mixed|string
 *
 * @deprecated use postDolibarrApi() instead
 */
function doRestPost($endpoint, $parameters = array(), $useGenericUser=false, $retourBrut=false) {

	// Format URL
	$url_api = dolispipFormatApiUrl($endpoint, $useGenericUser);
	//var_dump(json_encode($parameters));
	$curl = curl_init($url_api);
	//curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PUT");
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($curl, CURLOPT_POST, true);
	curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($parameters));
	$apiKey = getDolispipRestToken($useGenericUser);

	curl_setopt($curl, CURLOPT_HTTPHEADER, array('DOLAPIKEY: '.$apiKey, 'Content-Type: application/json'));

	$curl_response = curl_exec($curl);
	if ($curl_response === false) {
		curl_getinfo($curl);
		curl_close($curl);
		//die('error occured during curl exec. Additioanl info: ' . var_export($info));
		return false;
	}

	curl_close($curl);

	// On veut le retour brut
	if($retourBrut) {
		return $curl_response;
	}
	if(!$retourBrut)	{
		// Decode and convert into array
		$retour = json_decode($curl_response, true);
		if(isset($retour['error']) && isset($retour['error']['code'])) {
			switch($retour['error']['code']) {
				case "400":
					$retour = false;
					break;

				case "401":
					// Redirection vers page login
					$redirect = generer_url_public('moncompte','',true,0);
					include_spip('inc/headers');
					if(!lire_config('dolispip/activer_debug'))
						redirige_par_entete($redirect);
					$retour = '';
					break;
			}
		}
		return $retour;
	}
}

/**
 * Get a token for API by sending login and password request
 *
 * return string token
 */
function getRestToken($login, $pass) {
	include_spip('inc/session');
	$endpoint = 'login';
	$url_api = dolispipFormatApiUrl($endpoint);

	include_spip('inc/distant');
	$datas = array('login' => $login, 'password' => $pass );
	$res = recuperer_page($url_api, '', false, null, $datas, '', false, '', '');

	$result = json_decode($res);

	if($result->success->code == 200) {
		$token = $result->success->token;
		session_set('doli_api_key',$token);
	}

	return $token;
}

/**
 * Get the token for API access and check if already access.
 *
 */
function getDolispipRestToken($useGenericUser=false)
{
	include_spip('inc/session');
	include_spip('inc/config');

	// Generic user
	if($useGenericUser) {
		return lire_config('dolispip/dolibarr_api_key');
	} else {
		// Si jeton en session on le retourne
		$token = session_get('doli_api_key');
		if(strlen($token) > 0) {
			return $token;
		} 
	}
	return '';
}
