<?php

/*
 *
 *
 * Crée une commande à partir du panier
 *
 */
include_spip('inc/dolispip');
include_spip('inc/session');

/**
 * Récupère la liste des factures
 *
 */
function dolispip_get_list_factures() {
    $endpoint = 'invoices';
    return getDolibarrApi($endpoint);
}

/**
 * Récupère les infos d'une facture
 *
 * @param int $idFacture
 * @param boolean $use_user
 * @return boolean|mixed|string
 */
function dolispip_get_infos_facture($idFacture, $use_user=false) {
    $endpoint = 'invoices/'.$idFacture;
    $res = getDolibarrApi($endpoint, array(), $use_user);
    return $res;
}

/**
 * Récupère les lignes d'une facture
 *
 * @param int $idFacture
 * @param boolean $use_user
 * @return boolean|mixed|string
 */
function dolispip_get_lines_facture($idFacture, $use_user=false) {
    $endpoint = "invoices/$idFacture/lines";
    $res = getDolibarrApi($endpoint);
    return $res;
}

/**
 * Création d'une facture avec l'API Dolibarr
 *
 * @param array $invoice  Tableau avec les infos de facture
 */
function dolispip_creer_facture($invoice)
{
    $endpoint = 'invoices';
    $res = postDolibarrApi($endpoint, $invoice);
    return $res;
}

/**
 * Créer une facture depuis une commande
 * @param int $id_order
 * @return number
 */
function dolispip_creer_invoice_from_order($id_order) 
{
    // Le visiteur en cours
    $id_auteur = session_get('id_auteur') > 0 ? session_get('id_auteur') : 0;
    $socid = session_get('doli_socid');
    
    $endpoint = 'invoices/createfromorder/'.$id_order;
    $res = postDolibarrApi($endpoint, array(), true);
    
    if($id_facture = $res->id) {
        // Valid invoice
        $endpoint = 'invoices/'.$id_facture.'/validate';
        $res = postDolibarrApi($endpoint, array(), true, true);
        if($res->info->http_code === 200) {
            return $id_facture;
        }
    }
    return 0;
}
/**
 * Création d'une facture avec les webservices Dolibarr
 * @deprecated
 * @see dolispip_creer_invoice_from_order()
 */
function dolispip_creer_invoice($id_order)
{
    // Le visiteur en cours
    $id_auteur = session_get('id_auteur') > 0 ? session_get('id_auteur') : 0;
    $socid = session_get('doli_socid');

    $WS_DOL_URL = lire_config('dolispip/dolibarr_url').'webservices/server_invoice.php?wsdl';
    $WS_METHOD  = 'createInvoiceFromOrder';
    $params = array('id_order'=>$id_order);

    $result = doSoapCall($WS_METHOD, $params, $WS_DOL_URL);

    if (! $result &&  $result['result_code'] != 'OK') {
        //var_dump($soapclient);
        print '<h2>Erreur SOAP 1</h2>'.$soapclient->error_str;
        return false;
    } else {

        $id_facture = $result['id'];

        //Get invoice detail
        $WS_DOL_URL = lire_config('dolispip/dolibarr_url').'webservices/server_invoice.php?wsdl';
        $WS_METHOD  = 'getInvoice';
        $params = array('id' => $id_facture);

        $result = doSoapCall($WS_METHOD, $params, $WS_DOL_URL);
        $invoice = $result['invoice'];

        //Set as validate
        $invoice->status = 1;
        $WS_DOL_URL = lire_config('dolispip/dolibarr_url').'webservices/server_invoice.php?wsdl';
        $WS_METHOD  = 'updateInvoice';
        $params = array(
            'invoice'=>$invoice,
            'id_warehouse' => lire_config('dolispip_commandes/id_warehouse',0)
        );

        $result = doSoapCall($WS_METHOD, $params, $WS_DOL_URL);
    }

    return $result['id'];
}
