<?php
/**
 * Crée une propale à partir du panier
 *
 * param @thirparty array
 * return socid
 */
include_spip('inc/dolispip');


/**
 * Get category information
 *
 */
function dolispip_get_infos_categorie($categorieId) {
    if($categorieId >0) {
      $endpoint = 'categories/'.$categorieId;
      $res = getDolibarrApi($endpoint, array(), true);
      return $res;        
    }
    return '';
}

/**
 * Retourne les produits d'une catégorie
 *
 */
function dolispip_get_list_categorie_produits($categorieId) {
    if($categorieId >0) {
        // TODO : utiliser l'utilisateur actuellement connecté
        // permettrait d'utiliser les différentes règles de prix (multiprix / prix par client, …)
        $useGenericUser = true;

        $loadstock = lire_config('dolispip_produits/charge_stock');
        
        $endpoint = 'products';
        $params = array('mode' => 0, 'category' => "$categorieId", 'includestockdata' => $loadstock );
        return getDolibarrApi($endpoint, $params, $useGenericUser);
    }
    return '';
}

/**
 * Get list of sub-categories
 *
 */
function dolispip_get_list_subcategories($categorieParentId) {
    if($categorieParentId >0) {
        $endpoint = 'categories/';
        $res = getDolibarrApi($endpoint, array('sqlfilters' => 't.fk_parent='.$categorieParentId), true);
        return $res;
    }
    return '';
}

/*
 *
 * Retourne un objet à convertir en tableau si besoin avec get_object_vars()
 *
 * @param  string  $id       [description]
 * @param  string  $ref      [description]
 * @param  boolean $use_user [description]
 * @return [type]            [description]
 */

function dolispip_get_infos_produit($idProduit='', $ref = "", $useGenericUser=false) {

  include_spip('inc/dolispip');
  $res = array();

  $loadstock = lire_config('dolispip_produits/charge_stock');
  if (!empty($idProduit)) {
      $endpoint = 'products/'.$idProduit;
      $res = getDolibarrApi($endpoint, array('includestockdata' => $loadstock), $useGenericUser);
      return $res;
  }
  if (empty($idProduit) && !empty($ref)) {
      $endpoint = 'products/';
      $params = array('sqlfilters' => "t.ref LIKE '".$ref."'", 'includestockdata' => $loadstock);
      $res = getDolibarrApi($endpoint, $params, $useGenericUser);
      return $res[0];
  }
    return array();
}


/**
 * Converti durée
 *
 * @param  string $duree [description]
 * @return string        [description]
 */
function convert_duration($duree)
{
    $trans = array('d'=>'jour', 'm'=>'mois', 'y'=>'year');

    return $trans[$duree];

}


/**
 * fonctions pour les urls
 * Ajouter les règles suivantes dans le htaccess :
 *
 * # URL Rewriting pour dolispip
 * RewriteRule [a-z-\-]*-cat-([0-9]+)\.html$  spip.php?page=dolicategorie&id_doli=$1 [QSA,L]
 * RewriteRule [a-z-\-]*-prod-([0-9]+)\.html$ spip.php?page=doliproduit&id_doli=$1 [QSA,L]
 *
 * @param  int $id    ID objet
 * @param  string $label libellé
 * @param  string $type  type d'objet
 * @return string        url formatée
 */
function dolispip_url($id,$label,$type)
{
    $txt_url = str2url($label);
    $url = "".$txt_url."-".$type."-".$id.".html";
    return $url;

}

/**
 * Formate une chaine pour l'url rewriting
 *
 * @param  string $txt Chaîne à formatter
 * @return string      [description]
 */
function str2url($txt)
{
    $txt = trim($txt);
    $txt = strtolower($txt);
    $txt = html_entity_decode($txt);
    //$txt = iconv("ISO-8859-1", "UTF-8", $txt);
    $tab = array
    (
    "¥" => "Y", "µ" => "u", "À" => "A", "Á" => "A", "Â" => "A", "Ã" => "A",
    "Ä" => "A", "Å" => "A", "Æ" => "A", "Ç" => "C", "È" => "E", "É" => "E",
    "Ê" => "E", "Ë" => "E", "Ì" => "I", "Í" => "I", "Î" => "I", "Ï" => "I",
    "Ð" => "D", "Ñ" => "N", "Ò" => "O", "Ó" => "O", "Ô" => "O", "Õ" => "O",
    "Ö" => "O", "Ø" => "O", "Ù" => "U", "Ú" => "U", "Û" => "U", "Ü" => "U",
    "Ý" => "Y", "ß" => "s", "à" => "a", "á" => "a", "â" => "a", "ã" => "a",
    "ä" => "a", "å" => "a", "æ" => "a", "ç" => "c", "è" => "e", "é" => "e",
    "ê" => "e", "ë" => "e", "ì" => "i", "í" => "i", "î" => "i", "ï" => "i",
    "ð" => "o", "ñ" => "n", "ò" => "o", "ó" => "o", "ô" => "o", "õ" => "o",
    "ö" => "o", "ø" => "o", "ù" => "u", "ú" => "u", "û" => "u", "ü" => "u",
    "ý" => "y", "ÿ" => "y"
    );
    $txt = strtr("$txt", $tab);

    $txt = preg_replace('/[^a-z0-9-]/', '-', $txt);
    $txt = preg_replace('/-+/', '-', $txt);
    return $txt;
}

/**
 * [get_id_panier_encours description]
 *
 * @return void id panier si existant
 */
function get_id_panier_encours()
{
    include_spip('inc/dolipaniers');
    return paniers_id_dolipanier_encours();

}


/*
 * Function to know if an option is already in basket or not
 *
 */
function si_option_dans_panier($ref) {
    
    include_spip('base/abstract_sql');
    include_spip('inc/dolipaniers');
    include_spip('inc/session');
    
    $id_panier = paniers_id_dolipanier_encours();
    $ret = intval(sql_getfetsel(
        'id_dolipanier',
        'spip_dolipaniers_liens',
        array(
            'id_dolipanier = '.sql_quote($id_panier),
            'product_ref = '.sql_quote($ref)        )
        ));
    
    return $ret;
}


/*
 * Function to know if an option is already in basket or not
 *
 */
function si_doliproduit_dans_panier($ref) {
    
    include_spip('base/abstract_sql');
    include_spip('inc/dolipaniers');
    include_spip('inc/session');
    
    $id_panier = paniers_id_dolipanier_encours();
    
    $ret = intval(sql_getfetsel(
        'id_dolipanier',
        'spip_dolipaniers_liens',
        array(
            'id_dolipanier = '.sql_quote($id_panier),
            'product_ref = '.sql_quote($ref)
        )
    ));
    return $ret;
}

/**
 * Get options for a product
 * 
 * @param int $u    ID product Dolibarr
 * @return boolean|mixed|string
 */
function dolispip_get_options_product($u) {
    $endpoint = 'productoptions/'.$u;
    $res = getDolibarrApi($endpoint, array(), true);
    return $res;
}

/**
 * Get info for an optionnal product
 * 
 * @param int $id_product
 * @param int $id_option
 * @return boolean|mixed|string
 */
function dolispip_get_infos_prodoption($id_product, $id_option) {
    $endpoint = 'productoptions/price/'.$id_product.'/'.$id_option;
    $res = getDolibarrApi($endpoint, array(), true);
    return $res;
}


/**
 * Cherche une catégorie en fonction de son nom
 * 
 * @param string $name  Nom de la catégorie à rechercher
 * @param boolean $useGenericUser
 * @return array
 */
function dolispip_searchCategoryByName($name='', $useGenericUser=true) {
    
    include_spip('inc/dolispip');
    $res = array();
    
    if (!empty($name)) {
        $endpoint = 'categories';
        //$filter = "(t.label:LIKE:'".$name."')";
        //$params = array('sqlfilters' => $filter);
        $params = array('sqlfilters' => 't.label LIKE "'.$name.'"');
        $res = getDolibarrApi($endpoint, $params, $useGenericUser);
        return $res[0];
    }
    return array();
}

/**
 * Créer une categorie via API dolibarr
 * 
 * @param array $categorie Tableau decrivant la categorie
 * @return boolean|mixed|string
 */
function dolispip_produits_creer_categorie($categorie, $useGenericUser=true)
{
    include_spip('inc/dolispip');
    $endpoint = 'categories';
    $res = postDolibarrApi($endpoint, $categorie, $useGenericUser);
    return $res;
}

function dolispip_produits_creer_product($categorie, $useGenericUser = true)
{
    include_spip('inc/dolispip');
    $endpoint = 'products';
    $res = postDolibarrApi($endpoint, $categorie, $useGenericUser, true);
    if($res->info->http_code == 200)
        return true;
    else 
        return false;
        
}
