<?php
/**
 * Importer le calogue des produits dans dolibarr 
 *
 * 
 *
 * @plugin     dolispip-produits
 * @copyright  2018
 * @author     librethic
 * @licence    GPL 3
 * @package    DOLISPIP\produits\Formulaires
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) return;

/**
 * Chargement du formulaire 
 *
 * Déclarer les champs postés et y intégrer les valeurs par défaut
 *
 * @return Array
 *     Environnement du formulaire
 */
function formulaires_catalogue2dolispip_charger_dist(){
    
    $valeurs = array(
        'secteur_boutique' => 0,
        'importer_prop' => 0,
        'importer_refuse' => 0
    );
    
    // Admin sauf restreint
    if(session_get('statut') == '0minirezo' and !session_get('restreint')) {
        $valeurs['editable'] = true;
    }
    $valeurs['_saisie_en_cours'] = (_request('annuler')==null);
    
    return $valeurs;
}

/**
 * 
 * @return string[]
 */
function formulaires_catalogue2dolispip_verifier_dist() {
    $retour = array();
    if ( !lire_config('dolispip_produits/dolispip_cat_mere')) {
        $retour['message_erreur'] = 'Plugin non configuré (id cat mère)';
    }
    
    if ( ! $secteur_boutique = _request('secteur_boutique')) {
        $retour['secteur_boutique'] = "Manque rubrique";
    }
    
    if (!$retour && _request('confirmer') != 'on') {
        
        $retour['previsu'] = '<strong>Confirmer l\'export vers Dolibarr</strong>';
        $retour['message_erreur'] = 'Veuillez confirmer';
    }
    return $retour;
}

/**
 * 
 */
function formulaires_catalogue2dolispip_traiter_dist() {
    
    include_spip('inc/texte');

    $nbCreated = $nbProdExistant = 0; // Nombre de produit crées dans Dolibarr
    $id_secteur_boutique = _request('secteur_boutique');
    
    $produits_exclus_ids = _request('exclude_ids');
    if(!is_array($produits_exclus_ids)) $produits_exclus_ids = array();

    // Liste des catégories
    $liste_rubriques = sql_allfetsel('id_rubrique, titre, texte', 'spip_rubriques', 'id_parent='.$id_secteur_boutique );   
    
    // pour chaque categorie enfant du secteur
    foreach ($liste_rubriques as $rub) {
        
        // Vérif exist dans dolibarr
        $res = dolispip_searchCategoryByName(supprimer_numero($rub['titre']));
        if ($res->id > 0) {
            // Si oui : récup id pour produit
            $id_categorie_doli = $res->id;        
        } 
        else {
            //  si non création
            $categorie = array(
                'label' => supprimer_numero($rub['titre']), 
                'description'=> $rub['texte'],
                'fk_parent' => lire_config('dolispip_produits/dolispip_cat_mere'),
                'type' => 0
            );
            
            $id_categorie_doli = dolispip_produits_creer_categorie($categorie);
        }

        // Pour chaque produit de la catégorie
        $liste_produits = sql_allfetsel('id_produit,titre, reference, texte, prix_ht, taxe, statut, poids, largeur, longueur, hauteur', 'spip_produits', 'id_rubrique='.$rub['id_rubrique']);
        foreach ($liste_produits as $produit) {
            //var_dump($produit); exit;
            // Si pas exclus 
            if (! in_array($produit['id_produit'], $produits_exclus_ids)) {
                $searchProduct = dolispip_get_infos_produit('', $produit['reference'], true);
                //var_dump($searchProduct); exit; 
                // Si ref existe; continue
                if ($searchProduct->id > 0) {
                    ++$nbProdExistant;
                    continue;
                }
                else {
                // Si non : création 
                    $doliProduit = array(
                        'ref' => $produit['reference'],
                        'type' => 1,
                        'label' => $produit['titre'],
                        'description' => propre($produit['texte']),
                        'price' => $produit['prix_ht'],
                        'price_base_type' => 'HT',
                        'tva_tx' => $produit['taxe'],
                        'weight' => $produit['poids'],
                        'width' => $produit['longueur'],
                        'status' => ($produit['statut'] == 'publie' ? 1:0)
                    );
                    
                    $idProduitDoli = dolispip_produits_creer_product($doliProduit);
                    if ($idProduitDoli) {
                        ++$nbCreated;
                    }
                }
            }
        }
    }
    
    return array(
        'message_ok' => $nbCreated .' produits crées / ' . $nbProdExistant . ' existants',
        'editable' => false
    );
   
}
  