<?php
/***************************************************************************\
 *  SPIP, Systeme de publication pour l'internet                           *
 *                                                                         *
 *  Copyright (c) 2001-2008                                                *
 *  Arnaud Martin, Antoine Pitrou, Philippe Riviere, Emmanuel Saint-James  *
 *                                                                         *
 *  Ce programme est un logiciel libre distribue sous licence GNU/GPL.     *
 *  Pour plus de details voir le fichier COPYING.txt ou l'aide en ligne.   *
\***************************************************************************/
if (!defined("_ECRIRE_INC_VERSION")) { return;
}

include_spip('inc/utils');
include_spip('inc/config');

/**
 * Informer du droit de modifier ou non son login
 *
 * @param  string $serveur
 * @return bool
 *     toujours true pour un auteur cree dans SPIP
 */
function auth_dolibarr_autoriser_modifier_login($serveur='')
{
    if (strlen($serveur)) {
        return false; // les fonctions d'ecriture sur base distante sont encore incompletes
    }
    return true;
}

/**
 * Informer du droit de modifier ou non le pass
 *
 * @param  string $serveur
 * @return bool
 *    toujours true pour un auteur cree dans SPIP
 */
function auth_dolibarr_autoriser_modifier_pass($serveur='')
{
    if (strlen($serveur)) {
        return false; // les fonctions d'ecriture sur base distante sont encore incompletes
    }
    return true;
}


function auth_dolibarr_retrouver_login($login,$pass,$serveur='') 
{
    if (!strlen($login)) { return null; // pas la peine de requeter
    }

    // si l'auteur existe dans SPIP en tant que source PMB, c'est qu'il s'est deja authentifié une fois au moins
    // donc qu'il existe !
    if (sql_getfetsel(
        'id_auteur', 'spip_auteurs', array(
        'login='.sql_quote($login),
        'source='.sql_quote('dolibarr'),
        'statut<>'. sql_quote('5poubelle'))
    )
    ) {
         return $login;
    }

    // ici, c'est un login dolibarr "peut être"...
    // on demande à SPIP d'autoriser les connexions avec un mot de passe non
    // crypte lorsqu'on ne connait pas le login dans spip_auteurs
    //define ('_AUTORISER_AUTH_FAIBLE', true);
    

}

// Authentifie et si ok retourne le tableau de la ligne SQL de l'utilisateur
// Si risque de secu repere a l'installation retourne False
function auth_dolibarr_dist($login, $pass, $serveur='') 
{
    include_spip('inc/session');
    include_spip('inc/dolispip');
    include_spip("inc/config");

    // Authentification non activée, on quitte
    if(!lire_config('dolispip_tiers/activer_authentification') == 'oui') {
        return array();
    }

    // login inconnu, n'allons pas plus loin
    if (!$login) { return array();
    }
    
    // Securite
    if (!$login || !$pass) { return array();
    }
    // Test l'utilisateur dans Dolibarr avec requete WS en utilisant login et pass fourni
    include_spip('inc/config');

    include_spip('inc/dolispip');
    
    // récup des infos user, avec l'utilisateur générique
    $endpoint = 'users';
    $params = array('api_key' => $apiUserKey, 'sqlfilters' => "t.login like '$login'");
    $res = getDolibarrApi($endpoint, $params, true, true);
    
    if($res->info->http_code == 200) {
        $resp = $res->decode_response();
        if(count($resp) > 0 ) {
            $idUserToTest = $resp[0]->id;
        }
    }
    if($idUserToTest > 0) {
        $endpoint = 'users/'.$idUserToTest;
        $params = array('api_key' => $apiUserKey);
        $res = getDolibarrApi($endpoint, $params, true, true);
        if($res->info->http_code == 200) {
            $resp = $res->decode_response();
            $user_doli = $resp;
        } else {
            return '';
        }
    }
    else {
        //utilisateur inconnu
        return _T('dolispip_tiers:erreur_utilisateur_inconnu');
    }
    
    $endpoint = 'login';
    $parameters = array('login' => $login, 'password' => $pass, 'entity' => '', 'api_key' => false );
    $res = postDolibarrApi($endpoint, $parameters);
    // Si utilisateur loggé
    if(is_object($res->success) && $res->success->code == 200) {
        spip_log("Utilisateur logé avec API", 'dolispip'._LOG_DEBUG);
        $apiUserKey = $res->success->token;

        
        // Valeurs mises en sessions
        $credentials_dolibarr = array(
         'login' => $user_doli->login,
         //'doli_pass' => $pass,
         'doli_api_key' => $apiUserKey,
         //'doli_pass_crypted' => $passe_doli_crypted,
         'doli_fk_user' => $user_doli->id,
         'doli_fk_contact' => $user_doli->contactid,
         'doli_socid' => $user_doli->socid
        );
        spip_log("dolispip $login utilisateur connu de dolibarr et authentifié", 'dolispip');

        // requete sql sur spip_auteurs pour connaitre les infos
        // Si auteur trouvé on retourne ses infos
        if ($row = sql_fetsel("*", "spip_auteurs", "login=" . sql_quote($login)." AND source='dolibarr'")) {

               // mettre à jour infos auteur en provenance de Dolibarr
               $m = sql_updateq(
                   'spip_auteurs',
                   array(
                   'doli_socid'    => $user_doli->socid
                   ),
                   "id_auteur=".$row['id_auteur']
               );

               $row = array_merge($row, $credentials_dolibarr);

               // On envoi au pipeline
               $data = pipeline('dolibarr_authentifie', array("auteur" => $row,"statut"=>"existant"));
               return $data['auteur'];
        }

        // Pas d'auteur dans spip avec ce login, on le crée
        $id_auteur = creer_auteur_dolispip($user_doli);

        if($id_auteur > 0) {
             $row =  sql_fetsel("*", "spip_auteurs", "id_auteur=$id_auteur");
            
             $row = array_merge($row, $credentials_dolibarr);
            
             // On envoi au pipeline
             $data = pipeline('dolibarr_authentifie', array("auteur" => $row,"statut"=>"existant"));
             return $data['auteur'];
        }
    }
    else
    {
        spip_log("Erreur login user / code retour : ".$res->success->code, 'dolispip'._LOG_ERREUR);
        return '';
    }
}

function creer_auteur_dolispip($user) 
{

    $n = sql_insertq(
        'spip_auteurs', array(
        'source' => 'dolibarr',
        'nom' => $user->lastname." ".$user->firstname ,
        'login' => $user->login,
        'email' => $user->email,
        'statut' => lire_config('dolispip/statutclients_spip_dolibarr'),
        'bio' => $user->signature,
        'doli_socid' =>  $user->socid
        )
    );
    spip_log("dolispip Creation de l'auteur '$nom' dans spip_auteurs id->".$n, 'dolispip');
    return $n;
}

/**
 * Modifier le mot de passe de l'auteur sur le serveur concerne
 * en s'occupant du hash et companie
 *
 * @param  string $login
 * @param  string $new_pass
 * @param  int    $id_auteur
 * @param  string $serveur
 * @return bool
 */
function auth_dolibarr_modifier_pass($login, $new_pass, $id_auteur, $serveur='')
{
    include_spip('auth/spip');
    include_spip('inc/dolispip');
    if (is_null($new_pass) OR auth_spip_verifier_pass($login, $new_pass, $id_auteur, $serveur)!='') {
        return false;
    }

    if (!$id_auteur = intval($id_auteur)
        OR !sql_fetsel('login', 'spip_auteurs', 'id_auteur='.intval($id_auteur), '', '', '', '', $serveur)
    ) {
        return false;
    }

    // Appel API pour modifier le mot de passe
    // Trouve l'ID dans dolibarr
    $idUserDolibarr = 0;
    $endpoint = 'users';
    $params = array('api_key' => $apiUserKey, 'sqlfilters' => "t.login like '$login'");
    $res = getDolibarrApi($endpoint, $params, true, true);
    if($res->info->http_code == 200) {
        $resp = $res->decode_response();
        if(count($resp) > 0 ) {
            $idUserDolibarr = $resp[0]->id;
        }
    }
    
    if($idUserDolibarr > 0) {
        $endpoint = 'users/'.$idUserDolibarr;
        $parameters = array( 'pass' => $new_pass );
        $res = putDolibarrApi($endpoint, $parameters, true, true);
        if($res->info->http_code == 200) {
            return true;
        }
    }
    return false;
}

/**
 * informer sur un login
 * Ce dernier transmet le tableau ci-dessous a la fonction JS informer_auteur
 * Il est invoque par la fonction JS actualise_auteur via la globale JS
 * page_auteur=#URL_PAGE{informer_auteur} dans le squelette login
 * N'y aurait-il pas plus simple ?
 *
 * @param  array  $infos
 * @param  array  $row
 * @param  string $serveur
 * @return array
 */
function auth_dolibarr_informer_login($infos, $row, $serveur='')
{

    // pour la methode Dolibarr pas besoin des alea en plus
    $infos['alea_actuel'] = '';
    $infos['alea_futur'] = '';

    return $infos;
}

/**
 * Modifier le login d'un auteur Dolibarr
 *
 * @param  string $new_login
 * @param  int    $id_auteur
 * @param  string $serveur
 * @return bool
 */
function auth_dolibarr_modifier_login($new_login, $id_auteur, $serveur = '') 
{
    return true;   
}

/**
 * Verification de la validite d'un login Dolibarr
 *
 * @param  string $new_login
 * @param  int    $id_auteur
 *  si auteur existant deja
 * @param  string $serveur
 * @return string
 *  message d'erreur si login non valide, chaine vide sinon
 */
function auth_dolibarr_verifier_login($new_login, $id_auteur = 0, $serveur = '') 
{
    // login et mot de passe
    if (strlen($new_login)) {
        if (strlen($new_login) < _LOGIN_TROP_COURT) {
            return _T('info_login_trop_court_car_pluriel', array('nb' => _LOGIN_TROP_COURT));
        } else {
            $n = sql_countsel(
                'spip_auteurs',
                "login=" . sql_quote($new_login) . " AND id_auteur!=" . intval($id_auteur) . " AND statut!='5poubelle'", '', '',
                $serveur
            );
            if ($n) {
                return _T('info_login_existant');
            }
        }
    }
    
    return '';
}
